"""Suite Enumerations: Enumerations for the Finder
Level 1, version 1

Generated from /System/Library/CoreServices/Finder.app
AETE/AEUT resource version 0/144, language 0, script 0
"""

import aetools
import MacOS

_code = 'tpnm'

from StdSuites.Type_Names_Suite import *
class Enumerations_Events(Type_Names_Suite_Events):

    pass

_Enum_earr = {
    'not_arranged' : 'narr',    #
    'snap_to_grid' : 'grda',    #
    'arranged_by_name' : 'nama',        #
    'arranged_by_modification_date' : 'mdta',   #
    'arranged_by_creation_date' : 'cdta',       #
    'arranged_by_size' : 'siza',        #
    'arranged_by_kind' : 'kina',        #
    'arranged_by_label' : 'laba',       #
}

_Enum_ecvw = {
    'icon_view' : 'icnv',       #
    'list_view' : 'lsvw',       #
    'column_view' : 'clvw',     #
}

_Enum_edfm = {
    'Mac_OS_format' : 'dfhf',   #
    'Mac_OS_Extended_format' : 'dfh+',  #
    'UFS_format' : 'dfuf',      #
    'NFS_format' : 'dfnf',      #
    'audio_format' : 'dfau',    #
    'ProDOS_format' : 'dfpr',   #
    'MS_2d_DOS_format' : 'dfms',        #
    'ISO_9660_format' : 'df96', #
    'High_Sierra_format' : 'dfhs',      #
    'QuickTake_format' : 'dfqt',        #
    'Apple_Photo_format' : 'dfph',      #
    'AppleShare_format' : 'dfas',       #
    'UDF_format' : 'dfud',      #
    'WebDAV_format' : 'dfwd',   #
    'FTP_format' : 'dfft',      #
    'Packet_2d_written_UDF_format' : 'dfpu',    #
    'unknown_format' : 'df??',  #
}

_Enum_elsv = {
    'name_column' : 'elsn',     #
    'modification_date_column' : 'elsm',        #
    'creation_date_column' : 'elsc',    #
    'size_column' : 'elss',     #
    'kind_column' : 'elsk',     #
    'label_column' : 'elsl',    #
    'version_column' : 'elsv',  #
    'comment_column' : 'elsC',  #
}

_Enum_ipnl = {
    'General_Information_panel' : 'gpnl',       #
    'Sharing_panel' : 'spnl',   #
    'Memory_panel' : 'mpnl',    #
    'Preview_panel' : 'vpnl',   #
    'Application_panel' : 'apnl',       #
    'Languages_panel' : 'pklg', #
    'Plugins_panel' : 'pkpg',   #
    'Name__26__Extension_panel' : 'npnl',       #
    'Comments_panel' : 'cpnl',  #
    'Content_Index_panel' : 'cinl',     #
}

_Enum_isiz = {
    'mini' : 'miic',    #
    'small' : 'smic',   #
    'large' : 'lgic',   #
}

_Enum_lvic = {
    'small_icon' : 'smic',      #
    'large_icon' : 'lgic',      #
}

_Enum_priv = {
    'read_only' : 'read',       #
    'read_write' : 'rdwr',      #
    'write_only' : 'writ',      #
    'none' : 'none',    #
}

_Enum_sodr = {
    'normal' : 'snrm',  #
    'reversed' : 'srvs',        #
}

_Enum_vwby = {
    'conflicts' : 'cflc',       #
    'existing_items' : 'exsi',  #
    'small_icon' : 'smic',      #
    'icon' : 'iimg',    #
    'name' : 'pnam',    #
    'modification_date' : 'asmo',       #
    'size' : 'ptsz',    #
    'kind' : 'kind',    #
    'comment' : 'comt', #
    'label' : 'labi',   #
    'version' : 'vers', #
    'creation_date' : 'ascd',   #
    'small_button' : 'smbu',    #
    'large_button' : 'lgbu',    #
    'grid' : 'grid',    #
    'all' : 'kyal',     #
}


#
# Indices of types declared in this module
#
_classdeclarations = {
}

_propdeclarations = {
}

_compdeclarations = {
}

_enumdeclarations = {
    'earr' : _Enum_earr,
    'ecvw' : _Enum_ecvw,
    'edfm' : _Enum_edfm,
    'elsv' : _Enum_elsv,
    'ipnl' : _Enum_ipnl,
    'isiz' : _Enum_isiz,
    'lvic' : _Enum_lvic,
    'priv' : _Enum_priv,
    'sodr' : _Enum_sodr,
    'vwby' : _Enum_vwby,
}
